@@ -25,9 +25,6 @@ my $grid_cells;
 # Local transposition.
 my $xpose = 0;
 
-# Chord type for this song, used to detect mixing types.
-my $chordtype;
-
 # Used chords, in order of appearance.
 my @used_chords;
 
@@ -35,9 +32,16 @@ my @used_chords;
 my @chorus;
 my $chorus_xpose = 0;
 
+# Memorized chords.
+my %memchords;			# all sections
+my $memchords;			# current section
+my $memcrdinx;			# chords tally
+my $memorizing;			# if memorizing (a.o.t. recalling)
+
 # Keep track of unknown chords, to avoid dup warnings.
 my %warned_chords;
 
+my $re_chords;			# for chords
 my $re_meta;			# for metadata
 
 my @labels;			# labels used
@@ -48,6 +52,8 @@ my $no_substitute;
 
 my $diag;			# for diagnostics
 
+sub ::break() {}
+
 sub parsefile {
     my ( $self, $filename, $options ) = @_;
     $options //= {};
@@ -80,18 +86,34 @@ sub parse_song {
 
     $song = App::Music::ChordPro::Song->new
       ( source => { file => $diag->{file}, line => 1 + $$linecnt },
+	system => App::Music::ChordPro::Chords::get_parser,
 	structure => "linear",
       );
 
     $xpose = 0;
-    $grid_arg = '1+4x4+1';
+    $grid_arg = [ 4, 4, 1, 1 ];	# 1+4x4+1
     $in_context = $def_context;
     @used_chords = ();
     %warned_chords = ();
-    undef $chordtype;
+    %memchords = ();
     App::Music::ChordPro::Chords::reset_song_chords();
     @labels = ();
 
+    # Pre-fill meta data, if any.
+    if ( $options->{meta} ) {
+	while ( my ($k, $v ) = each( %{ $options->{meta} } ) ) {
+	    $song->{meta}->{$k} = [ $v ];
+	}
+    }
+
+    # Build regexp to split out chords.
+    if ( $::config->{settings}->{memorize} ) {
+	$re_chords = qr/(\[.*?\]|\^)/;
+    }
+    else {
+	$re_chords = qr/(\[.*?\])/;
+    }
+
     # Build regex for the known metadata items.
     if ( $::config->{metadata}->{keys} ) {
 	$re_meta = '^(' .
@@ -100,6 +122,7 @@ sub parse_song {
 	$re_meta = qr/$re_meta/;
     }
     else {
+	# HUH?
 	undef $re_meta;
     }
 
@@ -132,8 +155,8 @@ sub parse_song {
 			text => $_ )
 	      unless
 	    $options->{_legacy}
-	      ? $self->global_directive( $1, 1 )
-	      : $self->directive($1);
+	      ? $self->global_directive( $1, $options, 1 )
+	      : $self->directive( $1, $options );
 	    next;
 	}
 
@@ -174,11 +197,12 @@ sub parse_song {
 	$diagrams = $::config->{diagrams}->{show};
     }
 
+    my $target = $::config->{settings}->{transcode} || $song->{system};
     if ( $diagrams =~ /^(user|all)$/
-	 && defined($chordtype)
-	 && $chordtype =~ /^[RN]$/ ) {
+	 && !App::Music::ChordPro::Chords::Parser->get_parser($target,1)->has_diagrams ) {
 	$diag->{orig} = "(End of Song)";
-	do_warn("Chord diagrams suppressed for Nashville/Roman chords");
+	do_warn( "Chord diagrams suppressed for " .
+		 ucfirst($target) . " chords" ) unless $options->{silent};
 	$diagrams = "none";
     }
 
@@ -188,7 +212,7 @@ sub parse_song {
 
 	if ( $diagrams eq "user" ) {
 	    @used_chords =
-	    grep { safe_chord_info($_)->{origin} == 1 } @used_chords;
+	    grep { safe_chord_info($_)->{origin} eq "user" } @used_chords;
 	}
 
 	if ( $::config->{diagrams}->{sorted} ) {
@@ -203,8 +227,22 @@ sub parse_song {
 	  };
     }
 
-    # Global transposition.
-    $song->transpose( $options->{transpose} );
+
+    my $xp = $options->{transpose};
+    my $xc = $::config->{settings}->{transcode};
+    if ( $xc && App::Music::ChordPro::Chords::Parser->get_parser($xc,1)->movable ) {
+	if ( $song->{meta}->{key}
+	     && ( my $i = App::Music::ChordPro::Chords::parse_chord($song->{meta}->{key}->[0]) ) ) {
+	    $xp = - $i->{root_ord};
+	    delete $song->{meta}->{key};
+	}
+	else {
+	   $xp = 0;
+	}
+    }
+
+    # Global transposition and transcoding.
+    $song->transpose( $xp, $xc );
 
     # $song->structurize;
 
@@ -225,27 +263,14 @@ sub add {
 sub chord {
     my ( $self, $c ) = @_;
     return $c unless length($c);
+    return $c if $c =~ /^\*/;
     my $parens = $c =~ s/^\((.*)\)$/$1/;
 
     my $info = App::Music::ChordPro::Chords::identify($c);
-    if ( $info->{system} ) {
-	if ( defined $chordtype ) {
-	    if ( $chordtype ne $info->{system} ) {
-		$chordtype = $info->{system};
-		do_warn("Mixed chord systems detected in song");
-	    }
+    unless ( $info->{system} ) {
+	if ( $info->{error} && ! $warned_chords{$c}++ ) {
+	    do_warn( $info->{error} ) unless $c =~ /^n\.?c\.?$/i;
 	}
-	else {
-	    $chordtype = $info->{system};
-	}
-    }
-    elsif ( $info->{warning} && ! $warned_chords{$c}++ ) {
-	do_warn("Mysterious chord: $c")
-	  unless $c =~ /^n\.?c\.?$/i;
-    }
-    elsif ( $info->{error} && ! $warned_chords{$c}++ ) {
-	do_warn("Unrecognizable chord: $c")
-	  unless $c =~ /^n\.?c\.?$/i;
     }
 
     # Local transpose, if requested.
@@ -276,25 +301,59 @@ sub cxpose {
 sub decompose {
     my ($self, $line) = @_;
     $line =~ s/\s+$//;
-    my @a = split(/(\[.*?\])/, $line, -1);
+    my @a = split( $re_chords, $line, -1);
 
-    die(msg("Illegal line")."\n") unless @a; #### TODO
+#    die(msg("Illegal line")."\n") unless @a; #### TODO
 
-    if ( @a == 1 ) {
+    if ( @a <= 1 ) {
 	return ( phrases => [ $line ] );
     }
 
     shift(@a) if $a[0] eq "";
-    unshift(@a, '[]') if $a[0] !~ /^\[/;
-
+    unshift(@a, '[]') if $a[0] !~ $re_chords;
 
     my @phrases;
     my @chords;
     while ( @a ) {
-	my $t = shift(@a);
-	$t =~ s/^\[(.*)\]$/$1/;
-	push(@chords, $self->chord($t));
+	my $chord = shift(@a);
 	push(@phrases, shift(@a));
+
+	# Normal chords.
+	if ( $chord =~ s/^\[(.*)\]$/$1/ && $chord ne "^" ) {
+	    push(@chords, $self->chord($chord));
+	    if ( $memchords ) {
+		if ( $memcrdinx == 0 ) {
+		    $memorizing++;
+		}
+		if ( $memorizing ) {
+		    push( @$memchords, $chords[-1] );
+		}
+		$memcrdinx++;
+	    }
+	}
+
+	# Recall memorized chords.
+	elsif ( $memchords ) {
+	    if ( $memcrdinx == 0 && @$memchords == 0 ) {
+		do_warn("No chords memorized for $in_context");
+		push( @chords, $chord );
+		undef $memchords;
+	    }
+	    elsif ( $memcrdinx >= @$memchords ) {
+		do_warn("Not enough chords memorized for $in_context");
+		push( @chords, $chord );
+	    }
+	    else {
+		push( @chords, $memchords->[$memcrdinx]);
+	    }
+	    $memcrdinx++;
+	}
+
+	# Not memorizing.
+	else {
+	    #do_warn("No chords memorized for $in_context");
+	    push( @chords, $chord );
+	}
     }
 
     return ( phrases => \@phrases, chords  => \@chords );
@@ -364,6 +423,9 @@ sub decompose_grid {
 	elsif ( $_ eq '%%' ) {
 	    $_ = { symbol => $_, class => "repeat2" };
 	}
+	elsif ( $_ eq "/" ) {
+	    $_ = { symbol => $_, class => "slash" };
+	}
 	elsif ( $_ eq "." ) {
 	    $_ = { symbol => $_, class => "space" };
 	    $nbt++;
@@ -393,7 +455,7 @@ sub dir_split {
 }
 
 sub directive {
-    my ($self, $d) = @_;
+    my ( $self, $d, $options ) = @_;
     my ( $dir, $arg ) = dir_split($d);
 
     # Context flags.
@@ -407,19 +469,24 @@ sub directive {
 	do_warn("Already in " . ucfirst($in_context) . " context\n")
 	  if $in_context;
 	$in_context = $1;
-	$arg = $grid_arg if $in_context eq "grid" && $arg eq "";
-	if ( $in_context eq "grid" && $arg &&
+	@chorus = (), $chorus_xpose = 0 if $in_context eq "chorus";
+	if ( $in_context eq "grid" && $arg eq "" && $grid_arg ) {
+	    $self->add( type => "set",
+			name => "gridparams",
+			value => $grid_arg );
+	}
+	elsif ( $in_context eq "grid" && $arg &&
 	     $arg =~ m/^
 		       (?: (\d+) \+)?
 		       (\d+) (?: x (\d+) )?
 		       (?:\+ (\d+) )?
-		       $/x ) {
+		       (?:\s+ (.*)? )? $/x ) {
 	    do_warn("Invalid grid params: $arg (must be non-zero)"), return
 	      unless $2;
+	    $grid_arg = [ $2, $3, $1, $4 ];
 	    $self->add( type => "set",
 			name => "gridparams",
-			value => [ $2, $3, $1, $4 ] );
-	    $grid_arg = $arg;
+			value =>  [ @$grid_arg, $5||"" ] );
 	    $grid_cells = [ $2 * ( $3//1 ), ($1//0), ($4//0) ];
 	}
 	elsif ( $arg && $arg ne "" ) {
@@ -432,13 +499,21 @@ sub directive {
 	    do_warn("Garbage in start_of_$1: $arg (ignored)\n")
 	      if $arg;
 	}
-	@chorus = (), $chorus_xpose = 0 if $in_context eq "chorus";
+
+	# Enabling this always would allow [^] to recall anyway.
+	# Feature?
+	if ( $::config->{settings}->{memorize} ) {
+	    $memchords = $memchords{$1} //= [];
+	    $memcrdinx = 0;
+	    $memorizing = 0;
+	}
 	return 1;
     }
     if ( $dir =~ /^end_of_(\w+)$/ ) {
 	do_warn("Not in " . ucfirst($1) . " context\n")
 	  unless $in_context eq $1;
 	$in_context = $def_context;
+	undef $memchords;
 	return 1;
     }
     if ( $dir =~ /^chorus$/i ) {
@@ -446,9 +521,28 @@ sub directive {
 	    do_warn("{chorus} encountered while in $in_context context -- ignored\n");
 	    return 1;
 	}
+
+	# Clone the chorus so we can modify the label, if required.
+	my $chorus =
+	  @chorus ? App::Music::ChordPro::Config::clone(\@chorus) : [];
+
+	if ( @$chorus && $arg && $arg ne "" ) {
+	    if ( $chorus->[0]->{type} eq "set" && $chorus->[0]->{name} eq "label" ) {
+		$chorus->[0]->{value} = $arg;
+	    }
+	    else {
+		unshift( @$chorus,
+			 { type => "set",
+			   name => "label",
+			   value => $arg,
+			   context => "chorus",
+			 } );
+	    }
+	    push( @labels, $arg );
+	}
 	$self->add( type => "rechorus",
-		    @chorus
-		    ? ( "chorus" => App::Music::ChordPro::Config::clone(\@chorus),
+		    @$chorus
+		    ? ( "chorus" => $chorus,
 			"transpose" => $xpose - $chorus_xpose )
 		    : (),
 		  );
@@ -475,21 +569,21 @@ sub directive {
 
     # Comments. Strictly speaking they do not belong here.
 
+    my $comment;
     if ( $dir =~ /^(?:comment|c|highlight)$/ ) {
-	$self->add( type => "comment", $self->cdecompose($arg),
-		    orig => $arg );
-	return 1;
+	$comment = "comment";
     }
-
-    if ( $dir =~ /^(?:comment_italic|ci)$/ ) {
-	$self->add( type => "comment_italic", $self->cdecompose($arg),
-		    orig => $arg );
-	return 1;
+    elsif ( $dir =~ /^(?:comment_italic|ci)$/ ) {
+	$comment = "comment_italic";
     }
-
-    if ( $dir =~ /^(?:comment_box|cb)$/ ) {
-	$self->add( type => "comment_box", $self->cdecompose($arg),
-		    orig => $arg );
+    elsif ( $dir =~ /^(?:comment_box|cb)$/ ) {
+	$comment = "comment_box";
+    }
+    if ( $comment ) {
+	my %res = $self->cdecompose($arg);
+	$self->add( type => $comment, %res,
+		    orig => $arg )
+	  unless exists($res{text}) && $res{text} =~ /^[ \t]*$/;
 	return 1;
     }
 
@@ -548,14 +642,8 @@ sub directive {
     # Metadata extensions (legacy). Should use meta instead.
     # Only accept the list from config.
     if ( $re_meta && $dir =~ $re_meta ) {
-	if ( $xpose && $1 eq "key" ) {
-	    $arg = App::Music::ChordPro::Chords::transpose( $arg, $xpose );
-	}
-	if ( $1 eq "capo" && $song->{meta}->{capo} ) {
-	    do_warn("Multiple capo settings may yield surprising results.");
-	}
-	push( @{ $song->{meta}->{$1} }, $arg );
-	return 1;
+	$arg = "$1 $arg";
+	$dir = "meta";
     }
 
     # More metadata.
@@ -563,25 +651,29 @@ sub directive {
 	if ( $arg =~ /([^ :]+)[ :]+(.*)/ ) {
 	    my $key = lc $1;
 	    my $val = $2;
-	    if ( $xpose && $key eq "key" ) {
-		$val = App::Music::ChordPro::Chords::transpose( $val, $xpose );
+	    if ( $key eq "key" ) {
+		$val =~ s/[\[\]]//g;
+		my $xp = $xpose;
+		$xp += $options->{transpose} if $options->{transpose};
+		$val = App::Music::ChordPro::Chords::transpose( $val, $xp )
+		  if $xp;
 	    }
-	    if ( $key eq "capo" && $song->{meta}->{capo} ) {
-		do_warn("Multiple capo settings may yield surprising results.");
+	    elsif ( $key eq "capo" ) {
+		do_warn("Multiple capo settings may yield surprising results.")
+		  if $song->{meta}->{capo};
 	    }
-	    if ( $re_meta && $key =~ $re_meta ) {
-		# Known.
-		push( @{ $song->{meta}->{$key} }, $val );
+	    elsif ( $key eq "duration" ) {
+		$val = duration($val);
 	    }
-	    elsif ( $::config->{metadata}->{strict} ) {
+
+	    if ( $::config->{metadata}->{strict}
+		 && ! ( $re_meta && $key =~ $re_meta ) ) {
 		# Unknown, and strict.
 		do_warn("Unknown metadata item: $key");
 		return;
 	    }
-	    else {
-		# Allow.
-		push( @{ $song->{meta}->{$key} }, $val );
-	    }
+
+	    push( @{ $song->{meta}->{$key} }, $val );
 	}
 	else {
 	    do_warn("Incomplete meta directive: $d\n");
@@ -590,17 +682,31 @@ sub directive {
 	return 1;
     }
 
-    return 1 if $self->global_directive( $d, 0 );
+    return 1 if $self->global_directive( $d, $options, 0 );
 
     # Warn about unknowns, unless they are x_... form.
     do_warn("Unknown directive: $d\n") unless $d =~ /^x_/;
     return;
 }
 
+sub duration {
+    my ( $dur ) = @_;
+
+    if ( $dur =~ /(?:(?:(\d+):)?(\d+):)?(\d+)/ ) {
+	$dur = $3 + ( $2 ? 60 * $2 :0 ) + ( $1 ? 3600 * $1 : 0 );
+    }
+    my $res = sprintf( "%d:%02d:%02d",
+		       int( $dur / 3600 ),
+		       int( ( $dur % 3600 ) / 60 ),
+		       $dur % 60 );
+    $res =~ s/^[0:]+//;
+    return $res;
+}
+
 my %propstack;
 
 sub global_directive {
-    my ($self, $d, $legacy ) = @_;
+    my ($self, $d, $options, $legacy ) = @_;
     my ( $dir, $arg ) = dir_split($d);
 
     # Song / Global settings.
@@ -684,6 +790,10 @@ sub global_directive {
     # More private hacks.
     if ( $d =~ /^([-+])([-\w.]+)$/i ) {
 	return if $legacy;
+	if ( $2 eq "dumpmeta" ) {
+	    use Data::Dumper;
+	    warn(Dumper($song->{meta}));
+	}
 	$self->add( type => "set",
 		    name => $2,
 		    value => $1 eq "+" ? 1 : 0,
@@ -844,7 +954,7 @@ sub global_directive {
 	    my $ci;
 	    if ( $res->{frets} || $res->{base} || $res->{fingers} ) {
 		$ci = { name => $res->{name},
-			base => $res->{base} ? $res->{base} : 0,
+			base => $res->{base} ? $res->{base} : 1,
 			strings => $res->{frets},
 			$res->{fingers} ? ( fingers => $res->{fingers} ) : (),
 		      };
@@ -880,6 +990,11 @@ sub global_directive {
 		App::Music::ChordPro::Chords::add_unknown_chord( $res->{name} );
 	    }
 	}
+	elsif ( 1 ) {
+#	    $res->{base} = 0;
+# push( @{$song->{define}}, $res );
+	    App::Music::ChordPro::Chords::add_unknown_chord( $res->{name} );
+	}
 	else {
 	    unless ( App::Music::ChordPro::Chords::chord_info($res->{name}) ) {
 		do_warn("Unknown chord: $res->{name}\n");
@@ -894,11 +1009,11 @@ sub global_directive {
 }
 
 sub transpose {
-    my ( $self, $xpose ) = @_;
-    return unless $xpose;
+    my ( $self, $xpose, $xcode ) = @_;
+    return unless $xpose || $xcode;
 
     foreach my $song ( @{ $self->{songs} } ) {
-	$song->transpose($xpose);
+	$song->transpose( $xpose, $xcode );
     }
 }
 
@@ -941,42 +1056,42 @@ sub new {
 }
 
 sub transpose {
-    my ( $self, $xpose ) = @_;
+    my ( $self, $xpose, $xcode ) = @_;
 
     # Transpose meta data (key).
     if ( exists $self->{meta} && exists $self->{meta}->{key} ) {
 	foreach ( @{ $self->{meta}->{key} } ) {
-	    $_ = $self->xpchord( $_, $xpose );
+	    $_ = $self->xpchord( $_, 0, $xcode );
 	}
     }
 
     # Transpose song chords.
     if ( exists $self->{chords} ) {
 	foreach my $item ( $self->{chords} ) {
-	    $self->_transpose( $item, $xpose );
+	    $self->_transpose( $item, $xpose, $xcode );
 	}
     }
 
     # Transpose body contents.
     if ( exists $self->{body} ) {
 	foreach my $item ( @{ $self->{body} } ) {
-	    $self->_transpose( $item, $xpose );
+	    $self->_transpose( $item, $xpose, $xcode );
 	}
     }
 }
 
 sub _transpose {
-    my ( $self, $item, $xpose ) = @_;
+    my ( $self, $item, $xpose, $xcode ) = @_;
     $xpose //= 0;
 
     if ( $item->{type} eq "rechorus" ) {
 	return unless $item->{chorus};
 	for ( @{ $item->{chorus} } ) {
-	    $self->_transpose( $_, $xpose + $item->{transpose} );
+	    $self->_transpose( $_, $xpose + $item->{transpose}, $xcode );
 	}
 	return;
     }
-    return unless $xpose;
+    return unless $xpose || $xcode;
 
     if ( $item->{type} eq "songline" ) {
 	# Prevent chords to be autovivified.
@@ -984,7 +1099,7 @@ sub _transpose {
 	return unless exists $item->{chords};
 
 	foreach ( @{ $item->{chords} } ) {
-	    $_ = $self->xpchord( $_, $xpose );
+	    $_ = $self->xpchord( $_, $xpose, $xcode );
 	}
 	return;
     }
@@ -992,7 +1107,7 @@ sub _transpose {
     if ( $item->{type} =~ /^comment/ ) {
 	return unless $item->{chords};
 	foreach ( @{ $item->{chords} } ) {
-	    $_ = $self->xpchord( $_, $xpose );
+	    $_ = $self->xpchord( $_, $xpose, $xcode );
 	}
 	return;
     }
@@ -1000,16 +1115,16 @@ sub _transpose {
     if ( $item->{type} eq "gridline" ) {
 	foreach ( @{ $item->{tokens} } ) {
 	    next unless $_->{class} eq "chord";
-	    $_->{chord} = $self->xpchord( $_->{chord}, $xpose );
+	    $_->{chord} = $self->xpchord( $_->{chord}, $xpose, $xcode );
 	}
 	if ( $item->{margin} && exists $item->{margin}->{chords} ) {
 	    foreach ( @{ $item->{margin}->{chords} } ) {
-		$_ = $self->xpchord( $_, $xpose );
+		$_ = $self->xpchord( $_, $xpose, $xcode );
 	    }
 	}
 	if ( $item->{comment} && exists $item->{comment}->{chords} ) {
 	    foreach ( @{ $item->{comment}->{chords} } ) {
-		$_ = $self->xpchord( $_, $xpose );
+		$_ = $self->xpchord( $_, $xpose, $xcode );
 	    }
 	}
 	return;
@@ -1017,18 +1132,18 @@ sub _transpose {
 
     if ( $item->{type} eq "diagrams" ) {
 	foreach ( @{ $item->{chords} } ) {
-	    $_ = $self->xpchord( $_, $xpose );
+	    $_ = $self->xpchord( $_, $xpose, $xcode );
 	}
 	return;
     }
 }
 
 sub xpchord {
-    my ( $self, $c, $xpose ) = @_;
-    return $c unless length($c) && $xpose;
+    my ( $self, $c, $xpose, $xcode ) = @_;
+    return $c unless length($c) && ($xpose || $xcode);
     return $c if ref $c;
     my $parens = $c =~ s/^\((.*)\)$/$1/;
-    my $xc = App::Music::ChordPro::Chords::transpose( $c, $xpose );
+    my $xc = App::Music::ChordPro::Chords::transpose( $c, $xpose, $xcode );
     $xc ||= $c;
     return $parens ? "($xc)" : $xc;
 }
